# Copyright (C) 2012 Sergio Rossell
#
# This script is part of the EXAMO software
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
# 

"""
After running the _01_findZeroAndHighFreqRxns.py script the file: 'freqBasedRxns_description.pkl' should appear in the 'data' directory.

This script is the first part of our implementation of Jerby's Model Building Algorthm (MBA). This script generates functioning constraint-based models by heuristically prunning reactions outside the high-frequency-reaction set. Warning MBA is computationally very expesive. MBA is also an embarrassingly parallel problemm. The user should specify 'numProc', the number of parallel process to be initiated, and 'numRep' the number of times each process is repeated. Each process will write the list of reactions in the candidate model in a directory called 'data/mbaRxns'

"""
from sys import path; path.append('./examoModules/')
import os
import time

from examoModules import *

################################################################################
# INPUTS

description = 'glc_15_85'# tag to be added to output files]
numProc = 1# 100    #number of parallel processes used
numRep = 1# 10      #number of times each process is repeated

#threshold above which a flux is considered to be larger than zero
activityThreshold = 1E-10

md = importPickle('data/iMM904_blkRxnsDeleted_dict.pkl')

# Importing model information
fbr = importPickle('data/freqBasedRxns_%s.pkl' % description)

fOutMbaCandRxns = 'data/mbaCandRxns/mbaCandRxns_%s.pkl'
################################################################################
# STATEMENTS
# Instantiating CbModel 
m0 = CbModel(md['S'], md['idSp'], md['idRs'], md['lb'], md['ub'], md['rxns'],
        md['genes'])

# Deleting zero frequency creactions
m = deleteCbmRxns(m0, fbr['zfr'])

# making sure that all hfr reactions are active to begin with
act = findActiveRxns(m, 1E-10, fbr['hfr'])
cH = fbr['hfr'] & act
# Making sure that biomass can be produced
cH.add('R_biomass_published')

print '%i processes and %i repetitions' % (numProc, numRep)
for i in range(numProc):
    pid = os.fork()
    if pid == 0:
        for j in range(numRep):
            locTime = time.localtime()
            pid = os.getpid()
            timeStr = '%i%02i%02i%02i%02i%02i' % locTime[:6]
            tag = '%s_%s_%s' % (description, pid, timeStr)
            try:
                cr = iterativePrunning(m, cH, activityThreshold, m.idRs)
                exportPickle(cr, fOutMbaCandRxns % tag)
            except:
                print 'gurobi error, no solution found %s'  % description
        os._exit(0)

