# Copyright (C) 2012 Sergio Rossell
#
# This script is part of the EXAMO software
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
# 
"""
After running _02_minimizeNetwork_part_A.py MBA-based candidate reactions should have been written into the 'data/mbaRxns' directory. This script selects candidate MBA reaction lists based on the description tag and creates and EXAMO environoment-specific model.
"""
import os
from copy import copy
from numpy import array

from sys import path; path.append('./examoModules/')
from examoModules import *

################################################################################
# INPUTS

description = 'glc_15_85'
mbaCandRxnsDirectory = 'data/mbaCandRxns/'

#threshold above which a flux is considered to be larger than zero
activityThreshold = 1E-10

md = importPickle('data/iMM904_blkRxnsDeleted_dict.pkl')

# Importing model information
fbr = importPickle('data/freqBasedRxns_%s.pkl' % description)

fOutModel = 'data/iMM904_examo_%s.pkl'

################################################################################
# STATEMENTS
# Instantiating CbModel 
m = CbModel(md['S'], md['idSp'], md['idRs'], md['lb'], md['ub'], md['rxns'],
        md['genes'])

# Retrieving MBA candidate reaction lists
files = os.popen('ls %s | grep %s' % (mbaCandRxnsDirectory, description)).read().splitlines()


candRxnsMinusHFR = []
allRs = set()
rxnSets = []
for fn in files:
    l = set(importPickle(mbaCandRxnsDirectory + fn)) - fbr['hfr']
    rxnSets.append(importPickle(mbaCandRxnsDirectory + fn))
    allRs.update(l)
    candRxnsMinusHFR.append(l)

# checking the degree of overlap in MBA candidate reaction lists
overlap = []
l = range(len(candRxnsMinusHFR))
while l:
    ind = l.pop()
    for i in l:
        a = candRxnsMinusHFR[ind]
        b = candRxnsMinusHFR[i]
        overlap.append(float(len(a & b))/(max(len(a), len(b))))
overlap = array(overlap)

# Quantifying the number of times a rxn is among the candidate models
rxnFreq = {}
for rs in rxnSets:
    for rxn in rs:
        try:
            rxnFreq[rxn] += 1
        except KeyError:
            rxnFreq[rxn] = 1

freq = {}#{frequency : list of reactions}
for rxn in rxnFreq:
    try:
        freq[rxnFreq[rxn]].add(rxn)
    except KeyError:
        freq[rxnFreq[rxn]] = set([rxn])
orderedFreq = freq.keys()
orderedFreq.sort(reverse = True)
mRxns = copy(fbr['hfr'])
for num in orderedFreq:
    mRxns.update(freq[num])
    excRxns = set(m.idRs) - mRxns
    m1 = deleteCbmRxns(m, excRxns)
    act = findActiveRxns(m1, 1E-10, fbr['hfr'])
    if len(fbr['hfr'] - act) == 0:
        break
m2 = deleteCbmRxns(m, excRxns)

################################################################################
# OUTPUTS
# Reporting the degree of overlap in MBA candidate reaction lists
print 'mean overlap ', overlap.mean()
print 'std dev overlap ', overlap.std()   

# exporting model after minumizaetion
exportPickle(m2, fOutModel % description)

rev = [0 if val >= 0 else 1 for val in m.lb]
gene2rxn = {}
rxns = {}
for rxn in m2.rxns:
    gene2rxn[rxn] = m2.rxns[rxn]['genes']
    rxns[rxn] = m2.rxns[rxn]
mDict = {
        'S' : m2.S,
        'idSp' : m2.idSp,
        'idRs' : m2.idRs,
        'lb' : m2.lb,
        'ub' : m2.ub,
        'rev' : rev,
        'genes' : m2.genes,
        'gene2rxn' : gene2rxn,
        'rxns' : rxns,
        'descrip' : 'iMM904 examo %s' % description}
#exporting model dictionary
exportPickle(mDict, fOutModel % (description + '_dict'))
