# Copyright (C) 2012 Sergio Rossell
#
# This script is part of the EXAMO software
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
# 
"""
After creating an EXAMO environment-specific model, this scirpt minimizes the sum of all fluxes in the model subject to the constraint that all HFR should be active. The resulting flux distribution is exported as a comma-separated-value file.
"""

import csv
from sys import path; path.append('./examoModules/')
from examoModules import *

################################################################################
# INPUTS

#description = 'glc_15_85'
description = 'eth_15_85'
eps = 1E-3
activityThreshold = 1E-10

fModelExamo = 'data/iMM904_examo_%s_dict.pkl'
fFreqBasedRxns = 'data/freqBasedRxns_%s.pkl'

fOutMetabState = 'data/metabolicState_%s.csv'

################################################################################
# STATEMENTS

hfr = importPickle(fFreqBasedRxns % description)['hfr']
md = importPickle(fModelExamo % description)
m0 = CbModel(md['S'], md['idSp'], md['idRs'], md['lb'], md['ub'], md['rxns'],
        md['genes'])
#forcing biomass production
m0.lb[m0.idRs.index('R_biomass_published')] = 100.
#minimizing the sum of fluxes
#hfr = hfr & set(m0.idRs)
m = MipSeparateFwdRev_gurobi(m0, hfr, eps)
m.initMipGurobi()
m.minSumFluxes_gurobi()
nz = getNzRxnsGurobi(m)[1]

# reporting the flux distribution obtained
f = open(fOutMetabState % description, 'w')
csv.writer(f).writerows(nz)
f.close()

